function [BW_vis, BW_th, out_vis, out_th, dur_vis, dur_th] = extract_hands(vis_pth, ...
    th_pth, seg_thr, trans_rot, trans_shift_x, trans_shift_y, trans_pad)

% [BW_vis, BW_th, out_vis, out_th, dur_vis, dur_th] =
% extract_hands(vis_pth, th_pth, seg_thr, trans_rot, trans_shift_x,
% trans_shift_y, trans_pad)
% 
% This function segments hands in the VIS and TH spectrum.
% 
% vis_pth           - path to the visible image; if the input file is in
%                     the *.bmt format, TH and VIS images will be extracted
%                     from this file
% th_pth            - path to the thermal image
% seg_thr           - segmentation threshold (default: 95/255); if set to
%                     -1, the threshold will be calculated using K-means;
%                     if set to 0, the threshold will be calculated using
%                     Otsu's method
% trans_rot         - angle of rotation (default: 1)
% trans_shift_x     - shift in the X direction (default: 17)
% trans_shift_y     - shift in the Y direction (default: 30)
% trans_pad         - padding on the borders (default: -4)
% 
% BW_vis            - output mask for VIS image
% BW_th             - output mask for TH image
% out_vis           - segmented VIS image
% out_th            - segmented TH image
% dur_vis           - duration of segmentation of VIS image
% dur_th            - duration of segmentation of TH image
% 
% Example 1: Segment hand stored in the *.bmt file using threshold 110
% [BW_vis, BW_th, out_vis, out_th, dur_vis, dur_th] = extract_hands('IV_301.BMT', [], 110);
% 
% Example 2: Segment hand stored in the *.bmp files using K-means
% [BW_vis, BW_th, out_vis, out_th, dur_vis, dur_th] = extract_hands('visimg.bmp', 'thimg.bmp', -1);

if((nargin < 3) || (isempty(seg_thr)))
    seg_thr = 95/255;
end
if((nargin < 4) || (isempty(trans_rot)))
    trans_rot = 1;
end
if((nargin < 5) || (isempty(trans_shift_x)))
    trans_shift_x = 17;
end
if((nargin < 6) || (isempty(trans_shift_y)))
    trans_shift_y = 30;
end
if((nargin < 7) || (isempty(trans_pad)))
    trans_pad = -4;
end

%% Load and pre-process images
src_th = [];
src_vis = [];
[pathstr, name, ext] = fileparts(vis_pth);

if(strcmpi(ext, '.bmt'))
    src_th = imread(vis_pth);
    
    tmp = [num2str(round(rand(1)*1e5)) '.tmp'];
    while(exist(tmp,'file'))
        tmp = [num2str(round(rand(1)*1e5)) '.tmp'];
    end
    
    tmp_pth = fopen(tmp, 'w');
    vis_bin = fopen(vis_pth, 'r');
    img = fread(vis_bin, 'uint8');
    fwrite(tmp_pth, img(153723:end));
    
    fclose(tmp_pth);
    fclose(vis_bin);
    
    src_vis = imread(tmp);
    delete(tmp);
else
    src_th = imread(th_pth);
    src_vis = imread(vis_pth);
end

img_th = src_th;
img_vis = src_vis;

if(size(img_th,3) ~= 1)
    img_th = rgb2gray(img_th);
end

if(size(img_vis,3) ~= 1)
    img_vis = rgb2gray(img_vis);
end

%% Get mask from VIS
tic;
BW_vis = [];

if(seg_thr > 0)
    BW = im2bw(img_vis, seg_thr);
    BW_vis = uint8(BW);
elseif(seg_thr == 0)
    % Otsu
    BW = im2bw(img_vis, graythresh(img_vis));
    BW_vis = uint8(BW);
else
    % Convert color space to CMY and LAB
    cform = makecform('srgb2lab');
    lab = applycform(src_vis,cform);
    cform = makecform('srgb2cmyk');
    cmyk = applycform(src_vis,cform);
    cmy = cmyk(:,:,1:3);

    nrows = size(lab,1);
    ncols = size(lab,2);
    
    color_mtx = zeros(nrows,ncols,6);
    color_mtx(:,:,1:3) = lab;
    color_mtx(:,:,4:6) = cmy;
    color_mtx = reshape(color_mtx,nrows*ncols,6);

    % Apply vector quantization
    BW_vis = kmeans(color_mtx, 2, 'Replicates', 3, ...
        'emptyaction', 'singleton');
    BW_vis = uint8(BW_vis);
    BW_vis = reshape(BW_vis,nrows,ncols);
end

% Apply median filter
if(seg_thr <= 0)
    BW_vis = medfilt2(BW_vis, 'indexed');
    sel = BW_vis == median(BW_vis(1,:));
    BW_vis(sel) = 0;
    BW_vis(~sel) = 1;
    BW_vis([1 end],:) = 0;
end

dur_vis = toc;
tic;

%% Get mask from TH
BW_th = BW_vis;

% Rotate mask
BW_th = imrotate(BW_th, trans_rot, 'crop');

% Padding
if(trans_pad > 0)
    num_row = size(BW_th,1)/gcd(size(BW_th,1),size(BW_th,2));
    num_col = size(BW_th,2)/gcd(size(BW_th,1),size(BW_th,2));
    
    BW_th = [zeros(trans_pad*num_row, size(BW_th,2)); BW_th; ...
        zeros(trans_pad*num_row, size(BW_th,2))];
    BW_th = [zeros(size(BW_th,1), trans_pad*num_col) ...
        BW_th zeros(size(BW_th,1), trans_pad*num_col)];
elseif(trans_pad < 0)
    num_row = size(BW_th,1)/gcd(size(BW_th,1),size(BW_th,2));
    num_col = size(BW_th,2)/gcd(size(BW_th,1),size(BW_th,2));
    
    BW_th = BW_th(abs(trans_pad)*num_row+1:end-abs(trans_pad)*num_row,:);
    BW_th = BW_th(:,abs(trans_pad)*num_col+1:end-abs(trans_pad)*num_col);
end

% Resize mask to TH size
BW_th = imresize(BW_th, size(img_th), 'nearest');

% Shift mask
if(trans_shift_x > 0)
    BW_th = [zeros(size(BW_th,1),trans_shift_x) ...
        BW_th(:,1:end-trans_shift_x)];
elseif(trans_shift_x < 0)
    BW_th = [BW_th(:,abs(trans_shift_x)+1:end) ...
        zeros(size(BW_th,1),abs(trans_shift_x))];
end

if(trans_shift_y > 0)
    BW_th = [BW_th(trans_shift_y+1:end,:); ...
        zeros(trans_shift_y,size(BW_th,2))];
elseif(trans_shift_y < 0)
    BW_th = [zeros(abs(trans_shift_y),size(BW_th,2)); ...
        BW_th(1:end-abs(trans_shift_y),:)];
end

dur_th = toc;

%% Segment images
out_th = src_th.*BW_th(:,:,ones(1,size(src_th,3)));
out_vis = src_vis.*BW_vis(:,:,ones(1,size(src_vis,3)));