function dist = get_img_dist(trans)

% dist = get_img_dist(trans)
% 
% This function firstly transforms segmented visible image and then
% calculates some kind of distance between transfromed VIS and TH image.
% 
% trans(1)  - angle of rotation (default: 1)
% trans(2)  - shift in the X direction (default: 17)
% trans(3)  - shift in the Y direction (default: 30)
% trans(4)  - padding on the borders (default: -4)
% 
% dist      - distance between the images
% 
% The variables below must be stored in get_img_dist_input.mat
% img_vis   - visible image
% img_th    - thermal image
% BW_vis    - mask of visible image
% dist_type - type of distance {2Dcorr, joint, euclidean, seuclidean,
%             cityblock, minkowski, chebychev, cosine, correlation,
%             spearman, hamming, jaccard}

%% Path and variables
if((nargin < 1) || (isempty(trans)))
    trans(1) = 1;
    trans(2) = 17;
    trans(3) = 30;
    trans(4) = -4;
else
    trans(2:end) = round(trans(2:end));
end

load('get_img_dist_input.mat');

%% Rules for the transformation
if(abs(trans(2)) > round(size(img_th,2)/4))
    dist = Inf;
    return
end
if(abs(trans(3)) > round(size(img_th,1)/4))
    dist = Inf;
    return
end

%% Extract VIS image and transform it
tran_vis = img_vis.*BW_vis;

% Rotate image
tran_vis = imrotate(tran_vis, trans(1), 'crop');

% Padding
if(trans(4) > 0)
    num_row = size(tran_vis,1)/gcd(size(tran_vis,1),size(tran_vis,2));
    num_col = size(tran_vis,2)/gcd(size(tran_vis,1),size(tran_vis,2));
    
    tran_vis = [zeros(trans(4)*num_row, size(tran_vis,2)); tran_vis; ...
        zeros(trans(4)*num_row, size(tran_vis,2))];
    tran_vis = [zeros(size(tran_vis,1), trans(4)*num_col) ...
        tran_vis zeros(size(tran_vis,1), trans(4)*num_col)];
elseif(trans(4) < 0)
    num_row = size(tran_vis,1)/gcd(size(tran_vis,1),size(tran_vis,2));
    num_col = size(tran_vis,2)/gcd(size(tran_vis,1),size(tran_vis,2));
    
    tran_vis = tran_vis(abs(trans(4))*num_row+1:end-abs(trans(4))*num_row,:);
    tran_vis = tran_vis(:,abs(trans(4))*num_col+1:end-abs(trans(4))*num_col);
end

% Resize image to TH size
tran_vis = imresize(tran_vis, size(img_th), 'nearest');

% Shift mask
if(trans(2) > 0)
    tran_vis = [zeros(size(tran_vis,1),trans(2)) ...
        tran_vis(:,1:end-trans(2))];
elseif(trans(2) < 0)
    tran_vis = [tran_vis(:,abs(trans(2))+1:end) ...
        zeros(size(tran_vis,1),abs(trans(2)))];
end

if(trans(3) > 0)
    tran_vis = [tran_vis(trans(3)+1:end,:); ...
        zeros(trans(3),size(tran_vis,2))];
elseif(trans(3) < 0)
    tran_vis = [zeros(abs(trans(3)),size(tran_vis,2)); ...
        tran_vis(1:end-abs(trans(3)),:)];
end

%% Calculate the distance between TH and transformed VIS image
m1 = size(img_th,1);
m2 = size(tran_vis,1);
n1 = size(img_th,2);
n2 = size(tran_vis,2);

if((m1 ~= m2) || (n1 ~= n2))
    [img_th, tran_vis] = same_size(img_th, tran_vis);
end

switch(dist_type)
    case '2Dcorr'
        [H1, H2, H12, M, C] = mutual_information(img_th, tran_vis);
        dist = -max(max(C));
    case 'joint'
        [H1, H2, H12, M, C] = mutual_information(img_th, tran_vis);
        dist = H12;
    case 'euclidean'
        dist = pdist2(double(reshape(img_th, 1, m1*n1)), ...
            double(reshape(tran_vis, 1, m2*n2)), dist_type);
    case 'seuclidean'
        dist = pdist2(double(reshape(img_th, 1, m1*n1)), ...
            double(reshape(tran_vis, 1, m2*n2)), dist_type);
    case 'cityblock'
        dist = pdist2(double(reshape(img_th, 1, m1*n1)), ...
            double(reshape(tran_vis, 1, m2*n2)), dist_type);
    case 'minkowski'
        dist = pdist2(double(reshape(img_th, 1, m1*n1)), ...
            double(reshape(tran_vis, 1, m2*n2)), dist_type);
    case 'chebychev'
        dist = pdist2(double(reshape(img_th, 1, m1*n1)), ...
            double(reshape(tran_vis, 1, m2*n2)), dist_type);
    case 'cosine'
        dist = pdist2(double(reshape(img_th, 1, m1*n1)), ...
            double(reshape(tran_vis, 1, m2*n2)), dist_type);
    case 'correlation'
        dist = pdist2(double(reshape(img_th, 1, m1*n1)), ...
            double(reshape(tran_vis, 1, m2*n2)), dist_type);
    case 'spearman'
        dist = pdist2(double(reshape(img_th, 1, m1*n1)), ...
            double(reshape(tran_vis, 1, m2*n2)), dist_type);
    case 'hamming'
        dist = pdist2(double(reshape(img_th, 1, m1*n1)), ...
            double(reshape(tran_vis, 1, m2*n2)), dist_type);
    case 'jaccard'
        dist = pdist2(double(reshape(img_th, 1, m1*n1)), ...
            double(reshape(tran_vis, 1, m2*n2)), dist_type);
    otherwise
        dist = pdist2(double(reshape(img_th, 1, m1*n1)), ...
            double(reshape(tran_vis, 1, m2*n2)), dist_type);
end