function [BW_vis, BW_th, out_vis, out_th, dur_vis, dur_th] = ...
    seg_hands_using_reg(vis_pth, th_pth, seg_thr, dist_type, trans_rot, ...
    trans_shift_x, trans_shift_y, trans_pad)

% [BW_vis, BW_th, out_vis, out_th, dur] = seg_hands_using_reg(vis_pth, th_pth, seg_thr, dist_type, trans_rot, trans_shift_x, trans_shift_y, trans_pad)
% 
% This function segments hands in the VIS and TH spectrum using image
% registration.
% 
% vis_pth           - path to the visible image; if the input file is in
%                     the *.bmt format, TH and VIS images will be extracted
%                     from this file
% th_pth            - path to the thermal image
% seg_thr           - segmentation threshold (default: 95/255); if set to
%                     -1, the threshold will be calculated using K-means;
%                     if set to 0, the threshold will be calculated using
%                     Otsu's method
% dist_type         - type of distance {2Dcorr, joint, euclidean,
%                     seuclidean, cityblock, minkowski, chebychev, cosine,
%                     correlation, spearman, hamming, jaccard} (default:
%                     correlation)
% trans_rot         - angle of rotation (default: 1)
% trans_shift_x     - shift in the X direction (default: 17)
% trans_shift_y     - shift in the Y direction (default: 30)
% trans_pad         - padding on the borders (default: -4)
% 
% BW_vis            - output mask for VIS image
% BW_th             - output mask for TH image
% out_vis           - segmented VIS image
% out_th            - segmented TH image
% dur_vis           - duration of segmentation of VIS image
% dur_th            - duration of segmentation of TH image
% 
% Example 1: Segment hand stored in the *.bmt file using threshold 110 and
% joint probability as a distance parameter
% [BW_vis, BW_th, out_vis, out_th, dur_vis, dur_th] = seg_hands_using_reg('IV_301.BMT', [], 110, 'joint');
% 
% Example 2: Segment hand stored in the *.bmp files using K-means and
% euclidean distance
% [BW_vis, BW_th, out_vis, out_th, dur_vis, dur_th] = seg_hands_using_reg('visimg.bmp', 'thimg.bmp', -1, 'euclidean');

%% Paths and variables
if((nargin < 2) || isempty(th_pth))
    th_pth = [];
end
if((nargin < 3) || isempty(seg_thr))
    seg_thr = 95/255;
end
if((nargin < 4) || isempty(dist_type))
    dist_type = 'correlation';
end
if((nargin < 5) || (isempty(trans_rot)))
    trans_rot = 1;
end
if((nargin < 6) || (isempty(trans_shift_x)))
    trans_shift_x = 17;
end
if((nargin < 7) || (isempty(trans_shift_y)))
    trans_shift_y = 30;
end
if((nargin < 8) || (isempty(trans_pad)))
    trans_pad = -4;
end

trans(1) = trans_rot;
trans(2) = trans_shift_x;
trans(3) = trans_shift_y;
trans(4) = trans_pad;

%% Load and pre-process images
src_th = [];
src_vis = [];
[pathstr, name, ext, versn] = fileparts(vis_pth);

if(strcmpi(ext, '.bmt'))
    src_th = imread(vis_pth);
    
    tmp = [num2str(round(rand(1)*1e5)) '.tmp'];
    while(exist(tmp,'file'))
        tmp = [num2str(round(rand(1)*1e5)) '.tmp'];
    end
    
    tmp_pth = fopen(tmp, 'w');
    vis_bin = fopen(vis_pth, 'r');
    img = fread(vis_bin, 'uint8');
    fwrite(tmp_pth, img(153723:end));
    
    fclose(tmp_pth);
    fclose(vis_bin);
    
    src_vis = imread(tmp);
    delete(tmp);
else
    src_th = imread(th_pth);
    src_vis = imread(vis_pth);
end

img_th = src_th;
img_vis = src_vis;

if(size(img_th,3) ~= 1)
    img_th = rgb2gray(img_th);
end

if(size(img_vis,3) ~= 1)
    img_vis = rgb2gray(img_vis);
end

%% Get mask from VIS
tic;
BW_vis = [];

if(seg_thr > 0)
    BW = im2bw(img_vis, seg_thr);
    BW_vis = uint8(BW);
elseif(seg_thr == 0)
    % Otsu
    BW = im2bw(img_vis, graythresh(img_vis));
    BW_vis = uint8(BW);
else
    % Convert color space to CMY and LAB
    cform = makecform('srgb2lab');
    lab = applycform(src_vis,cform);
    cform = makecform('srgb2cmyk');
    cmyk = applycform(src_vis,cform);
    cmy = cmyk(:,:,1:3);

    nrows = size(lab,1);
    ncols = size(lab,2);
    
    color_mtx = zeros(nrows,ncols,6);
    color_mtx(:,:,1:3) = lab;
    color_mtx(:,:,4:6) = cmy;
    color_mtx = reshape(color_mtx,nrows*ncols,6);

    % Apply vector quantization
    BW_vis = kmeans(color_mtx, 2, 'Replicates', 3, ...
        'emptyaction', 'singleton');
    BW_vis = uint8(BW_vis);
    BW_vis = reshape(BW_vis,nrows,ncols);
end

% Apply median filter
if(seg_thr <= 0)
    BW_vis = medfilt2(BW_vis, 'indexed');
    sel = BW_vis == median(BW_vis(1,:));
    BW_vis(sel) = 0;
    BW_vis(~sel) = 1;
    BW_vis([1 end],:) = 0;
end

dur_vis = toc;
tic;

%% Store the variables to *.mat file
save('get_img_dist_input.mat', 'dist_type', 'img_th', 'img_vis', 'BW_vis');

%% Find the optimal values
[new_trans, fval] = fminsearch(@get_img_dist, trans);
delete('get_img_dist_input.mat');
new_trans(2:end) = floor(new_trans(2:end));
dur_tmp = toc;

%% Extract the hands
[BW_vis, BW_th, out_vis, out_th, tmp_vis_dr, tmp_th_dr] = ...
    extract_hands(vis_pth, th_pth, seg_thr, new_trans(1), new_trans(2), ...
    new_trans(3), new_trans(4));

dur_th = dur_tmp + tmp_th_dr;